"use strict";

var WAVE_TABLE_LENGTH = 8192;

var PeriodicWaveDSP = {
  dspInit: function dspInit() {
    this._waveTable = null;
  },
  dspBuildWaveTable: function dspBuildWaveTable() {
    if (this._waveTable !== null) {
      return this._waveTable;
    }

    var waveTable = new Float32Array(WAVE_TABLE_LENGTH + 1);
    var real = this._real;
    var imag = this._imag;

    var maxAbsValue = 0;
    var periodicWaveLength = Math.min(real.length, 16);

    for (var i = 0; i < WAVE_TABLE_LENGTH; i++) {
      var x = i / WAVE_TABLE_LENGTH * Math.PI * 2;

      for (var n = 1; n < periodicWaveLength; n++) {
        waveTable[i] += real[n] * Math.cos(n * x) + imag[n] * Math.sin(n * x);
      }

      maxAbsValue = Math.max(maxAbsValue, Math.abs(waveTable[i]));
    }

    if (!this._constants && maxAbsValue !== 1) {
      for (var _i = 0; _i < WAVE_TABLE_LENGTH; _i++) {
        waveTable[_i] *= maxAbsValue;
      }
    }
    waveTable[WAVE_TABLE_LENGTH] = waveTable[0];

    this._waveTable = waveTable;

    return waveTable;
  }
};

module.exports = PeriodicWaveDSP;