"use strict";

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var WaveShaperNodeDSP = {
  dspInit: function dspInit() {
    this._kernels = [];
  },
  dspUpdateKernel: function dspUpdateKernel(curve, numberOfChannels) {
    if (curve === null) {
      numberOfChannels = 0;
    }
    if (numberOfChannels < this._kernels.length) {
      this._kernels.splice(numberOfChannels);
    } else if (this._kernels.length < numberOfChannels) {
      while (numberOfChannels !== this._kernels.length) {
        this._kernels.push(new WaveShaperKernel(this, this._kernels.length));
      }
    }

    switch (numberOfChannels) {
      case 0:
        this.dspProcess = this.dspProcess0;
        break;
      case 1:
        this.dspProcess = this.dspProcess1;
        break;
      case 2:
        this.dspProcess = this.dspProcess2;
        break;
      default:
        this.dspProcess = this.dspProcessN;
        break;
    }
  },
  dspProcess0: function dspProcess0() {
    this.outputs[0].bus.copyFrom(this.inputs[0].bus);
  },
  dspProcess1: function dspProcess1() {
    var inputBus = this.inputs[0].bus;
    var outputBus = this.outputs[0].bus;
    var inputs = inputBus.getChannelData();
    var outputs = outputBus.getMutableData();
    var kernels = this._kernels;

    kernels[0].process(inputs[0], outputs[0], this._curve, this.blockSize);
  },
  dspProcess2: function dspProcess2() {
    var inputBus = this.inputs[0].bus;
    var outputBus = this.outputs[0].bus;
    var inputs = inputBus.getChannelData();
    var outputs = outputBus.getMutableData();
    var blockSize = this.blockSize;
    var curve = this._curve;
    var kernels = this._kernels;

    kernels[0].process(inputs[0], outputs[0], curve, blockSize);
    kernels[1].process(inputs[1], outputs[1], curve, blockSize);
  },
  dspProcessN: function dspProcessN() {
    var inputBus = this.inputs[0].bus;
    var outputBus = this.outputs[0].bus;
    var inputs = inputBus.getChannelData();
    var outputs = outputBus.getMutableData();
    var blockSize = this.blockSize;
    var curve = this._curve;
    var kernels = this._kernels;

    for (var i = 0, imax = kernels.length; i < imax; i++) {
      kernels[i].process(inputs[i], outputs[i], curve, blockSize);
    }
  }
};

var WaveShaperKernel = function () {
  function WaveShaperKernel() {
    _classCallCheck(this, WaveShaperKernel);
  }

  _createClass(WaveShaperKernel, [{
    key: "process",
    value: function process(input, output, curve, inNumSamples) {
      for (var i = 0; i < inNumSamples; i++) {
        var x = (Math.max(-1, Math.min(input[i], 1)) + 1) * 0.5;
        var ix = x * (curve.length - 1);
        var i0 = ix | 0;
        var i1 = i0 + 1;

        if (curve.length <= i1) {
          output[i] = curve[curve.length - 1];
        } else {
          output[i] = curve[i0] + ix % 1 * (curve[i1] - curve[i0]);
        }
      }
    }
  }]);

  return WaveShaperKernel;
}();

module.exports = WaveShaperNodeDSP;