"use strict";

var audioDataUtil = require("./audioDataUtil");
var resampler = require("./resampler");

/**
 * @param {function}    decodeFn
 * @param {ArrayBuffer} audioData
 * @param {object}      opts
 * @return {Promise<AudioData>}
 */
function decode(decodeFn, audioData, opts) {
  opts = opts /* istanbul ignore next */ || {};

  return new Promise(function (resolve, reject) {
    return decodeFn(audioData, opts).then(function (result) {
      if (audioDataUtil.isAudioData(result)) {
        if (typeof opts.sampleRate === "number") {
          result = resampler.resample(result, opts.sampleRate);
        }
        return resolve(result);
      }
      return reject(new TypeError("Failed to decode audio data"));
    }, reject);
  });
}

module.exports = { decode: decode };