"use strict";

var nmap = require("nmap");

/**
 * @param {AudioData} audioData
 * @param {number}    sampleRate
 */
function resample(audioData, sampleRate) {
  if (audioData.sampleRate === sampleRate) {
    return audioData;
  }

  var rate = audioData.sampleRate / sampleRate;
  var numberOfChannels = audioData.channelData.length;
  var length = Math.round(audioData.channelData[0].length / rate);
  var channelData = nmap(numberOfChannels, function () {
    return new Float32Array(length);
  });

  for (var i = 0; i < length; i++) {
    var ix = i * rate;
    var i0 = ix | 0;
    var i1 = i0 + 1;
    var ia = ix % 1;

    for (var ch = 0; ch < numberOfChannels; ch++) {
      var v0 = audioData.channelData[ch][i0];
      var v1 = audioData.channelData[ch][i1];

      channelData[ch][i] = v0 + ia * (v1 - v0);
    }
  }

  return { numberOfChannels: numberOfChannels, length: length, sampleRate: sampleRate, channelData: channelData };
}

module.exports = { resample: resample };